/* delete.c
**
** Delete operations for the different database entry types
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "dbif.h"
#include "tools.h"
#include "vxerror.h"

#include "ParserData.h"
#include "procs.h"

static void do_delete (DBIF_FILE dbf, DBIF_CURSOR dbc)
{
    char *ename;
    dbif_rec *erec;
    if (!dbc) {
	vxerror_at (EX_SOFTWARE, 0, __FILE__, __LINE__ - 2,
		    "unable to select data set for user deletion!\n");
    }
    for (ename = dbif_first(dbc, &erec); ename; ename = dbif_next(dbc, &erec)) {
	dbif_del (dbf, ename);
	free (ename); dbif_free (erec);
    }
    dbif_end (dbc);
    return;
}

static int check_udel (ParserData *sd, char *username)
{
    dbif_rec *urec = dbif_get (sd->dbf, "::VXMAIL::");
    size_t aml = strlen (urec->folder);
    int rt;
    if (!strncmp (username, urec->folder, aml) && username[aml] == '@') {
	dbif_free (urec);
	ccerror (sd, "i can't delete a domain admin!");
	return 1;
    }
    dbif_free (urec);
    urec = dbif_get (sd->dbf, username);
    if (!urec) {
	ccerror (sd, "i can't delete a non-existing user!\n");
	return 1;
    }
    rt = dbif_rectype(urec);
    dbif_free (urec);
    if ((1 << rt) & CC_DENTRIES) { return 0; }
    ccerror (sd, "sorry, but this is no valid domain member ...\n");
    return 1;
}

static int check_ddel (ParserData *sd, char *domname)
{
    int rt;
    dbif_rec *drec = dbif_get (sd->dbf, domname);
    if (!drec) {
	ccerror (sd, "i can't delete a non-existing domain(link)!\n");
	return 1;
    }
    rt = dbif_rectype(drec);
    dbif_free (drec);
    if ((1 << rt) & CC_DOMLIKE) { return 0; }
    ccerror (sd, "sorry, but this is no domain ...\n");
    return 1;
}

/* Delete a user entry and all links pointing to it. */
int delete_user (ParserData *sd, char *username)
{
    DBIF_CURSOR dbc;
    char *em_addr;
    if (strchr (username, '@')) {
	em_addr = gdupl (__FILE__, __LINE__, username);
    } else {
	if (!sd->domainname) {
	    ccerror (sd, "no domain specified!");
	    return 0;
	}
	em_addr=gconc(__FILE__, __LINE__, username, "@", sd->domainname, NULL);
    }
    if (check_udel (sd, em_addr)) {
	gfree (em_addr);
	return 0;
    }
    dbc = dbif_select (sd->dbf, em_addr, st_udel);
    do_delete (sd->dbf, dbc);
    gfree (em_addr);
    return 1;
}

/* Delete a domain, all user entries in this domain and all links referring
** this domain or users in this domain.
*/
int delete_domain (ParserData *sd, char *domname)
{
    DBIF_CURSOR dbc;
    InitDB (sd);
    if (check_ddel (sd, domname)) { return 0; }
    dbc = dbif_select (sd->dbf, domname, st_ddel);
    do_delete (sd->dbf, dbc);
    return 1;
}

void delete_special (ParserData *sd, char *iname, int rt)
{
    dbif_rec *irec;
    char *rtname = (rt == RT_IP4TRANS ? "IPv4 translation" :
		    rt == RT_IP6TRANS ? "IPv6 translation" :
		    rt == RT_REDIRECT ? "domain redirection" :
					"VXTS password");
    InitDB (sd);
    irec = dbif_get (sd->dbf, iname);
    if (!irec) {
	ccerror (sd, "i can't delete a non-existing %s!", rtname);
	return;
    }
    if (dbif_rectype(irec) != rt) {
	dbif_free (irec);
	ccerror (sd, "sorry, but this is no %s!", rtname);
	return;
    }
    dbif_free (irec);
    dbif_del (sd->dbf, iname);
}
